#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# QtTube
#
# Copyright (C) 2007 Siegfried-Angel Gevatter Pujals <siggi.gevatter@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import sys, os
from PyQt4 import QtCore, QtGui

import preferences

from ui_main import Ui_MainWindow

class Gui(QtGui.QMainWindow):
	"""QtTube's main class."""
	
	def __init__(self, sysarg1='', parent=None):
		"""Initialize the GUI and many variables and load the configuration."""
		
		QtGui.QWidget.__init__(self, parent)
		self.ui = Ui_MainWindow()
		self.ui.setupUi(self)
		
		# Load configuration
		self.settings = preferences.loadSettings()
		self.ui.dest_folder.setText(self.settings.value('General/folder').toString())
		
		# Initialize variables
		self.dialogs = {
				'about': 			None,		# The instance of the "About..." window will be stored here
				'preferences':	None,		# The instance of the "Preferences" window will be stored here
			}
		self.videoInput = {
				'wasUrl':			None,		# The value videoUrl had last time self.urlChange() was called
				'wasId':				None,		# The value videoId had last time it changed
				'fromUrl':			None,		# Wheter there was an Id in videoUrl or not last time self.urlChange() was called
			}
		self.download =		None		# Holds the QHttp object
		self.downloadUrl =	None		# The url of the video that's currently being downloaded
		self.isMinimal =		None		# Whether the window is in "minimal" mode or not
		self.threadUrl =		None		# The thread that gets the download url from youtube-dl
		
		# Get the sizes for the «minimal» and the «expanded» views of the main window
		self.size = {
				'width':				{ 'min': self.minimumWidth(), 'max': self.maximumWidth() },
				'height':			{ 'min': self.minimumHeight(), 'max': self.maximumHeight() },
			}
		
		# Apply the sizes of the «minimal» view to the window and setup a timer
		if self.settings.value('Interface/startMinimal').toBool():
			self.isMinimal = True
			self.setFixedSize(QtCore.QSize(QtCore.QRect(0, 0, self.size['width']['min'], self.size['height']['min']).size()))
			if self.settings.value('Interface/effectExpand').toBool():
				self.timerExpand = QtCore.QTimer()
				QtCore.QObject.connect(self.timerExpand, QtCore.SIGNAL("timeout()"), self.expandViewSlow);
		
		# Connect signals with slots
		QtCore.QObject.connect(self.ui.actionExit, QtCore.SIGNAL("activated()"), self.close)
		QtCore.QObject.connect(self.ui.actionAbout, QtCore.SIGNAL("activated()"), self.showAbout)
		QtCore.QObject.connect(self.ui.actionPreferences, QtCore.SIGNAL("activated()"), self.showPreferences)
		
		QtCore.QObject.connect(self.ui.videoUrl, QtCore.SIGNAL("textChanged(QString)"), self.urlChange)
		QtCore.QObject.connect(self.ui.videoId, QtCore.SIGNAL("textChanged(QString)"), self.idChange)
		QtCore.QObject.connect(self.ui.dest_name, QtCore.SIGNAL("lostFocus()"), self.nameChange)
		QtCore.QObject.connect(self.ui.button_folder, QtCore.SIGNAL("clicked()"), self.browseFolder)
		QtCore.QObject.connect(self.ui.button_get, QtCore.SIGNAL("clicked()"), self.getVideo)
		QtCore.QObject.connect(self.ui.button_abort, QtCore.SIGNAL("clicked()"), self.downloadAbort)
		QtCore.QObject.connect(self.ui.videoUrl, QtCore.SIGNAL("returnPressed()"), self.getVideo)
		QtCore.QObject.connect(self.ui.videoId, QtCore.SIGNAL("returnPressed()"), self.getVideo)
		
		# If the application was called with a argument, and it is a URL, put it into videoUrl
		if sysarg1 is not '':
			if sysarg1[:7] == 'http://':
				self.ui.videoUrl.setText(sysarg1)
		
	def expandView(self):
		"""Expand the window, to show the «Progress» section too. If needed, calls expandViewSlow()."""
		
		if self.settings.value('Interface/effectExpand').toBool():
			self.timerExpand.start(7)
		else:
			self.setFixedSize(QtCore.QSize(QtCore.QRect(0, 0, self.size['width']['min'], self.size['height']['max']).size()))
		
		
	def expandViewSlow(self):
		"""Nice effect that slowly expands the window."""
		
		if self.height() < self.size['height']['max']:
			if (self.height() + 3) <= self.size['height']['max']:
				self.setFixedSize(QtCore.QSize(QtCore.QRect(0, 0, self.size['width']['min'], self.height() + 3).size()))
			else:
				self.setFixedSize(QtCore.QSize(QtCore.QRect(0, 0, self.size['width']['min'], self.size['height']['max']).size()))
		
		else:
			self.timerExpand.stop()
			self.isMinimal = None
		
	def idChange(self):
		"""Handles text changes in videoId."""
		
		if self.ui.videoId.text() != '':
			self.ui.button_get.setEnabled(True)
			
			if  os.path.splitext(str(self.ui.dest_name.text()))[0] == str(self.videoInput['wasId']) or self.ui.dest_name.text() == '':
				self.ui.dest_name.setText(self.ui.videoId.text() + '.flv')
			
		else:
			self.ui.button_get.setEnabled(False)
			
			if os.path.splitext(str(self.ui.dest_name.text()))[0] == str(self.videoInput['wasId']):
				self.ui.dest_name.setText('')
		
		self.videoInput['wasId'] = self.ui.videoId.text()
		
	def urlChange(self):
		""""Handles text changes in videoUrl."""
		
		# Automagically fix errors
		if self.ui.videoUrl.text()[0:4] == 'www.':
			self.ui.videoUrl.setText('http://' + self.ui.videoUrl.text())
		
		# Only continue if the URL seems valid
		if self.ui.videoUrl.text() != '' and str(self.ui.videoUrl.text())[0:7] != 'http://':
			if not self.ui.videoId.isEnabled():
				self.ui.videoId.setText('')
				self.ui.videoId.setEnabled(True)
			return 0
		
		try:
			self.ui.videoId.setText( 
				str(self.ui.videoUrl.text())[(str(self.ui.videoUrl.text()).index("v=") + 2):] )
			fromUrl = True
			try:
				self.ui.videoId.setText( 
					str(self.ui.videoId.text())[:str(self.ui.videoId.text()).index("&")] )
			except ValueError:
				pass
		except ValueError:
			fromUrl = False
		
		if self.videoInput['fromUrl'] and not fromUrl:
			self.ui.videoId.setText('')
		
		if self.ui.videoUrl.text() == '' and self.videoInput['wasUrl']:
			self.ui.videoId.setText('')
			self.videoInput['wasUrl'] = False
			
		if self.ui.videoUrl.text() and self.ui.videoId.text():
			self.ui.videoId.setEnabled(False)
			self.ui.button_get.setEnabled(True)
		elif self.ui.videoId.text() == '':
			self.ui.videoId.setEnabled(True)
			self.ui.button_get.setEnabled(False)
		
		if self.ui.videoId.text() != '' and ( os.path.splitext(str(self.ui.dest_name.text()))[0] == str(self.videoInput['wasId']) or self.ui.dest_name.text() == '' ):
			self.ui.dest_name.setText(self.ui.videoId.text() + '.fla')
		elif self.ui.videoId.text() == '' and os.path.splitext(str(self.ui.dest_name.text()))[0] == str(self.videoInput['wasId']):
			self.ui.dest_name.setText('')
		
		self.videoInput['fromUrl'] = fromUrl
		self.videoInput['wasUrl'] = self.ui.videoUrl.text()
		self.videoInput['wasId'] = self.ui.videoId.text()
		
	def nameChange(self):
		"""Handles changes in the name of the destination file."""
		
		# Add the extension, if it gets removed
		if self.ui.dest_name.text() != '' and str(self.ui.dest_name.text())[-4:] != '.fla':
			self.ui.dest_name.setText(self.ui.dest_name.text() + '.fla')
		
	def browseFolder(self):
		"""Shows a folder selection dialog and (if the user doesn't cancel it) sets the new destination in the input."""
		
		dest_folder = preferences.selectFolder(self, self.ui.dest_folder.text())
		
		if dest_folder:
			self.ui.dest_folder.setText(dest_folder)
		
	def getVideo(self):
		"""Starts a thread to youtube-dl, retrieves the download url, and does all other stuff needed to get the video."""
		
		if self.ui.button_get.isEnabled() is not True:
			return 0
		
		# Reset the data in «Progress»
		self.ui.progress_text.setText('')
		self.ui.progressBar.setValue(0)
		
		# Create the file-object where the video will be saved
		from os.path import join
		self.filename = os.path.join(str(self.ui.dest_folder.text()), str(self.ui.dest_name.text()))
		self.videofile = QtCore.QFile(self.filename)
		
		# TODO: Ask to continue an old download, if found (?).
		if self.videofile.exists():
			overwrite = QtGui.QMessageBox.warning(self, 
					self.tr("QtTube - Overwrite File?"),
					self.tr("There is already a file with this name. Are you sure you want to overwrite it?"),
					QtGui.QMessageBox.Yes | QtGui.QMessageBox.Abort,
					QtGui.QMessageBox.Abort,
				);
		
			if overwrite == QtGui.QMessageBox.Abort:
				return 0
		
		# Block all possible value changes by the user
		self.inputsEnable(False)
		
		self.ui.status.setText('Retrieving video URL...')
		
		from pal import GetDownloadUrl
		self.threadUrl = GetDownloadUrl(str(self.ui.videoId.text()), self.settings.value('General/script').toString())
		self.threadUrl.start()
		
		self.timerThread = QtCore.QTimer()
		QtCore.QObject.connect(self.timerThread, QtCore.SIGNAL("timeout()"), self.isThreadReady);
		self.timerThread.start(750);
		
	def isThreadReady(self):
		"""Checks if the `youtube-dl` thread is read and, if it is, calls download()."""
		
		if self.threadUrl.result is not None:
			self.timerThread.stop()
			self.downloadStart(self.threadUrl.result)
		
	def inputsEnable(self, enabled=True):
		"""Massively enables or disables all inputs and changes which buttons that are enabled or even visible."""
		
		self.ui.videoUrl.setEnabled(enabled)
		self.ui.videoId.setEnabled(enabled)
		self.ui.dest_folder.setEnabled(enabled)
		self.ui.button_folder.setEnabled(enabled)
		self.ui.dest_name.setEnabled(enabled)
		self.ui.button_get.setEnabled(enabled)
		
		if enabled is True:
			self.ui.stackButtons.setCurrentWidget(self.ui.p_get)
		else:
			self.ui.stackButtons.setCurrentWidget(self.ui.p_abort)
		
	def downloadStart(self, result):
		"""Gets the resulting output from `youtube-dl` and handles it."""
		
		try:
			self.downloadUrl = result[0]
			
		except IndexError:
			self.inputsEnable(True)
			self.beep()
			self.ui.status.setText('Failed. Couldn\'t get the URL of this video.')
			return 0
		
		# Change the status messages
		self.ui.status.setText('')
		self.ui.progress_text.setText('<i>' + self.downloadUrl + '</i>')
		
		# Expand the window, if needed
		if self.isMinimal is not None:
			self.expandView()
		
		# Load modulesneeded to perform the download
		from urllib import splithost
		from PyQt4 import QtNetwork
		
		# Start downloading the video
		downloadUrl = splithost(self.downloadUrl[5:])
		self.download = QtNetwork.QHttp(downloadUrl[0])
		self.download.get(downloadUrl[1], self.videofile)
		
		QtCore.QObject.connect(self.download, QtCore.SIGNAL("dataReadProgress(int, int)"), self.downloadUpdate)
		QtCore.QObject.connect(self.download, QtCore.SIGNAL("done(bool)"), self.downloadDone)
		
	def downloadUpdate(self, done, total):
		"""Gets the current status of the download and updates the GUI according to it."""
		
		self.ui.progressBar.setMaximum(total)
		self.ui.progressBar.setValue(done)
		
	def downloadDone(self, errors):
		"""Check if the download was successfull or not and updates the GUI according to that."""
		
		if self.download is None:
			return 0
		
		elif errors is True:
			self.ui.progress_text.setText('There was an error downloading the video. It may be incomplete.')
			self.beep()
		
		else:
			self.ui.progress_text.setText('Download successfully completed.')
			
		self.download.close()	# Close the connection to the server
		self.inputsEnable(True)	# Free all blocked input possibilities
		self.download = self.threadUrl = None
		
	def downloadAbort(self):
		"""Aborts the current download, if any."""
		
		if self.threadUrl is not None and self.threadUrl.isAlive():
			import signal
			os.kill(self.threadUrl.pipe.pid, signal.SIGKILL)
		
		if self.download is not None:
			QtCore.QObject.disconnect(self.download, QtCore.SIGNAL("done(bool)"), self.downloadDone)
			self.download.abort()
			self.ui.progress_text.setText('You have aborted the download.')
		
		self.inputsEnable(True)
		
	def beep(self):
		"""If the user has "Beep on error" enabled, a beep sound will be produced"""
		
		if self.settings.value('Interface/beep').toBool():
			print '\a'
		
	def showAbout(self):
		"""Displays the 'About...' dialog."""
		
		if self.dialogs['about'] is None:
			from ui_about import Ui_AboutWindow
			self.dialogs['about'] = QtGui.QDialog()
			self.dialogs['about'].ui = Ui_AboutWindow()
			self.dialogs['about'].ui.setupUi(self.dialogs['about'])
		
		self.dialogs['about'].show()
		self.dialogs['about'].activateWindow()
		
	def showPreferences(self):
		"""Displays the 'Preferences...' dialog."""
		
		if self.dialogs['preferences'] is None:
			self.dialogs['preferences'] = preferences.PreferencesGui(self, self.settings)
		
		self.dialogs['preferences'].show()
		self.dialogs['preferences'].activateWindow()
		
	def closeEvent(self, e):
		"""This function is called when the application is closed using one of the features that the window manager offers."""
		self.close()
		
	def close(self):
		"""Expands QMainWindow's close function, in order to add other stuff that has to be done before the program ends."""
		
		self.downloadAbort()
		
		if self.dialogs['about'] is not None:
			self.dialogs['about'].done(1)
		
		QtGui.QMainWindow.close(self)

if __name__ == "__main__":
	
	app = QtGui.QApplication(sys.argv)
	
	try:
		mainApp = Gui(sys.argv[1])
	except IndexError:
		mainApp = Gui()
	
	mainApp.show()
	
	try:
		app.exec_()
	except KeyboardInterrupt:
		sys.exit()
